const path = require("path");
const express = require("express");
const morgan = require("morgan");
const helmet = require("helmet");
const mongoSanitize = require("express-mongo-sanitize");
const xss = require("xss-clean");
const cookieParser = require("cookie-parser");
const compression = require("compression");
const cors = require("cors");
const session = require("express-session");
const passport = require("passport");
const dotenv = require("dotenv");
const cron = require("node-cron");

dotenv.config(); // Load environment variables from .env file

const User = require("./models/userModel");
const AppError = require("./utils/appError");
const globalErrorHandler = require("./controllers/errorController");
const userRouter = require("./routes/userRoutes");
const authRoutes = require("./routes/authRoutes");
const linkedinRoutes = require("./routes/linkedinAuthRoutes");
const interestRoutes = require("./routes/interestRoutes");
const connectionRoutes = require("./routes/connectionRoutes");
const lockerRoutes = require("./routes/lockRoutes");

const app = express();

// 1) GLOBAL MIDDLEWARES

app.enable("trust proxy");

// CORS
app.use(cors());
app.options("*", cors());

// Serving static files
app.use(express.static(path.join(__dirname, "public")));

// Serve static files from the "uploads" folder (for uploaded images)
app.use("/uploads", express.static(path.join(__dirname, "uploads")));

// Set security HTTP headers
app.use(helmet());

// Development logging
if (process.env.NODE_ENV === "development") {
  app.use(morgan("dev"));
}

// Body parser, reading data from the body into req.body
app.use(express.json({ limit: "10kb" }));
app.use(express.urlencoded({ extended: true, limit: "10kb" }));
app.use(cookieParser());

// Data sanitization against NoSQL query injection
app.use(mongoSanitize());

// Data sanitization against XSS
app.use(xss());

// Compression
app.use(compression());

app.use(
  session({
    secret: process.env.SESSION_SECRET || "your-secret-key",
    resave: false,
    saveUninitialized: false,
    cookie: {
      secure: process.env.NODE_ENV === "production",
      httpOnly: true,
      maxAge: 1000 * 60 * 60 * 24, // 1 day
    },
  })
);
// Cron job to check if any profile's visibility time has expired
cron.schedule("* * * * *", async () => {
  try {
    // Get users whose visibility time has passed and the profile is still hidden
    const users = await User.find({
      visibilityEndTime: { $lte: Date.now() }, // Check if current time is greater than visibilityEndTime
      isVisible: false, // Profile is currently hidden
    });

    users.forEach(async (user) => {
      // Make the profile visible again after the lock time has passed
      user.isVisible = true;
      user.visibilityEndTime = null; // Reset the visibility end time
      await user.save();

      console.log(`User ${user._id}'s profile is now visible again.`);
    });
  } catch (err) {
    console.error("Error checking profile visibility:", err);
  }
});

// Initialize passport for authentication
app.use(passport.initialize());
app.use(passport.session());

// Test middleware
app.use((req, res, next) => {
  req.requestTime = new Date().toISOString();
  next();
});

// 2) ROUTES

app.get("/", (req, res) => {
  res.status(200).send({
    success: true,
    message: "Welcome to App",
  });
});

// Mount routes
app.use("/api/auth", authRoutes); // Use the separated auth routes
app.use("/auth-linkedin", linkedinRoutes);
app.use("/api/v1/users", userRouter);
app.use("/api/v1/interests", interestRoutes);
app.use("/api/v1/connections", connectionRoutes);
app.use("/api/v1/lock", lockerRoutes);

app.get("/dashboard", (req, res) => {
  res.status(200).json({
    status: "Success",
    message: "User data saved to DB!!",
  });
});

// Handle undefined routes
app.all("*", (req, res, next) => {
  next(new AppError(`Can't find ${req.originalUrl} on this server!`, 404));
});

// Global error handling middleware
app.use(globalErrorHandler);

module.exports = app;
