const User = require("../models/userModel"); // User model
const Connection = require("../models/connectionModel"); // Connection model
const mongoose = require("mongoose");
const {
  createAndEmitNotification,
} = require("../utils/createAndEmitNotification");
// Socket.IO instance (make sure the server is set up to use socket.io)
let io;

exports.setSocketInstance = (socketIoInstance) => {
  io = socketIoInstance;
};

// Controller to send a connection request
exports.sendConnectionRequest = async (req, res) => {
  try {
    const userId = req.user._id; // Logged-in user ID from middleware
    const { targetUserId } = req.body; // Target user ID from the request body

    // Prevent sending request to themselves
    if (userId.toString() === targetUserId.toString()) {
      return res.status(400).json({
        status: "Fail",
        error: "You cannot send a connection request to yourself.",
      });
    }

    const targetUser = await User.findById(targetUserId);
    if (!targetUser) {
      return res.status(404).json({
        status: "Fail",
        error: "Target user not found.",
      });
    }

    const existingConnection = await Connection.findOne({
      $or: [
        { sender: userId, receiver: targetUserId },
        { sender: targetUserId, receiver: userId },
      ],
    });

    if (existingConnection) {
      return res.status(400).json({
        status: "Fail",
        error: "Connection request already sent or already connected.",
      });
    }

    const newConnectionRequest = new Connection({
      sender: userId,
      receiver: targetUserId,
      status: "pending",
    });

    await newConnectionRequest.save();

    // Create and emit notification for the target user
    await createAndEmitNotification(
      io, // Pass io here
      targetUserId, // The user receiving the notification
      `${req.user.firstName} ${req.user.lastName} has sent you a connection request.`,
      "connection", // Correct type
      `/connections/${newConnectionRequest._id}` // Optional link to view the request
    );

    res.status(200).json({
      status: "Success",
      message: "Connection request sent successfully.",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({
      status: "Fail",
      error: "An error occurred while sending the connection request.",
    });
  }
};

// Controller to accept a connection request
exports.acceptConnectionRequest = async (req, res) => {
  try {
    const userId = req.user._id; // Logged-in user ID from middleware
    const { connectionId } = req.body; // Connection request ID

    // Validate connection request
    const connection = await Connection.findById(connectionId).populate(
      "sender receiver",
      "firstName lastName email photo"
    );
    if (!connection) {
      return res.status(404).json({
        status: "Fail",
        message: "Connection request not found.",
      });
    }

    // Ensure only the receiver can accept the request
    if (connection.receiver._id.toString() !== userId.toString()) {
      return res.status(403).json({
        status: "Fail",
        message: "You are not authorized to accept this connection request.",
      });
    }

    // Update the connection status to 'accepted'
    connection.status = "accepted";
    await connection.save();

    // Perform atomic updates to both sender and receiver
    await Promise.all([
      User.findByIdAndUpdate(connection.sender._id, {
        $addToSet: { connections: connection.receiver._id },
        $inc: { totalConnections: 1 }, // Increment sender's total connections
      }),
      User.findByIdAndUpdate(connection.receiver._id, {
        $addToSet: { connections: connection.sender._id },
        $inc: { totalConnections: 1 }, // Increment receiver's total connections
      }),
    ]);

    // Emit real-time update to the sender
    io.to(connection.sender._id.toString()).emit("Connection Accepted", {
      connectionId: connection._id,
      receiver: {
        _id: connection.receiver._id,
        firstName: connection.receiver.firstName,
        lastName: connection.receiver.lastName,
        email: connection.receiver.email,
        photo: connection.receiver.photo,
      },
    });

    // Create and emit notification for the sender
    await createAndEmitNotification(
      io, // Pass io here
      connection.sender._id, // Sender is notified about the acceptance
      `${connection.receiver.firstName} ${connection.receiver.lastName} has accepted your connection request.`,
      "connection", // Correct type
      `/connections/${connection._id}` // Optional link to view the accepted connection
    );

    // Send response to the receiver
    res.status(200).json({
      status: "Success",
      message: "Connection request accepted successfully.",
      connection: {
        _id: connection._id,
        sender: {
          _id: connection.sender._id,
          firstName: connection.sender.firstName,
          lastName: connection.sender.lastName,
          email: connection.sender.email,
          photo: connection.sender.photo,
        },
        receiver: {
          _id: connection.receiver._id,
          firstName: connection.receiver.firstName,
          lastName: connection.receiver.lastName,
          email: connection.receiver.email,
          photo: connection.receiver.photo,
        },
        status: connection.status,
      },
    });
  } catch (error) {
    console.error("Error while accepting connection request:", error);
    res.status(500).json({
      status: "Fail",
      message: "An error occurred while accepting the connection request.",
    });
  }
};

// Controller to reject a connection request
exports.rejectConnectionRequest = async (req, res) => {
  try {
    const userId = req.user._id; // Logged-in user ID from middleware
    const { connectionId } = req.body; // Connection request ID

    // Find the connection request
    const connection = await Connection.findById(connectionId).populate(
      "sender receiver",
      "firstName lastName email photo"
    );
    if (!connection) {
      return res.status(404).json({
        status: "Fail",
        error: "Connection request not found.",
      });
    }

    // Only the receiver can reject the request
    if (connection.receiver._id.toString() !== userId.toString()) {
      return res.status(403).json({
        status: "Fail",
        error: "You are not authorized to reject this connection request.",
      });
    }

    // Delete the connection request from the database
    await Connection.findByIdAndDelete(connectionId);

    // Emit real-time update to the sender
    io.to(connection.sender._id.toString()).emit("Connection Rejected", {
      receiver: userId,
      connectionId,
    });

    // Create and emit notification for the sender
    await createAndEmitNotification(
      io, // Pass io here
      connection.sender._id, // Sender is notified about the rejection
      `${connection.receiver.firstName} ${connection.receiver.lastName} has rejected your connection request.`,
      "connection", // Correct type
      `/connections/${connectionId}` // Optional link to view the rejected connection
    );

    res.status(200).json({
      status: "Success",
      message: "Connection request rejected and removed successfully.",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({
      status: "Fail",
      error: "An error occurred while rejecting the connection request.",
    });
  }
};

// Controller to get all connections (accepted or pending)
exports.getUserConnections = async (req, res) => {
  try {
    const userId = req.user._id; // Logged-in user ID from middleware

    const queryObj = { ...req.query }; // Copy query parameters
    const excludedFields = ["page", "sort", "limit", "fields"];
    excludedFields.forEach((el) => delete queryObj[el]);

    // Filtering based on connection status (pending, accepted)
    if (queryObj.status) {
      queryObj.status = { $in: queryObj.status.split(",") }; // Handle multiple status values like "pending,accepted"
    }

    // Advanced filtering
    let queryStr = JSON.stringify(queryObj);
    queryStr = queryStr.replace(/\b(gte|gt|lte|lt)\b/g, (match) => `$${match}`);

    let query = Connection.find({
      $or: [{ sender: userId }, { receiver: userId }],
      ...JSON.parse(queryStr), // Apply the filtering
    })
      .populate("sender", "firstName lastName email photo") // Populate sender details
      .populate("receiver", "firstName lastName email photo") // Populate receiver details
      .sort({ createdAt: -1 }); // Default sorting by createdAt

    // Sorting
    if (req.query.sort) {
      const sortBy = req.query.sort.split(",").join(" ");
      query = query.sort(sortBy);
    }

    // Field limiting
    if (req.query.fields) {
      const fields = req.query.fields.split(",").join(" ");
      query = query.select(fields);
    } else {
      query = query.select("-__v"); // Exclude __v by default
    }

    // Pagination
    const page = req.query.page * 1 || 1;
    const limit = req.query.limit * 1 || 10;
    const skip = (page - 1) * limit;
    query = query.skip(skip).limit(limit);

    if (req.query.page) {
      const numConnections = await Connection.countDocuments({
        $or: [{ sender: userId }, { receiver: userId }],
      });
      if (skip >= numConnections) throw new Error("This page does not exist");
    }

    // Execute query
    const connections = await query;

    // Send response
    res.status(200).json({
      status: "success",
      results: connections.length,
      data: {
        connections,
      },
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: "Fail",
      error: "Something went wrong while fetching connections",
    });
  }
};
