const InterestTab = require("../models/interestsModel");

// Create a new Interest Tab
exports.createInterestTab = async (req, res) => {
  try {
    const { topCategory, interests } = req.body;

    // Check if category is provided
    if (!topCategory || !interests || !Array.isArray(interests)) {
      return res.status(400).json({
        status: "fail",
        error: "Category and interests must be provided.",
      });
    }

    // Create a new Interest Tab
    const newInterestTab = new InterestTab({ topCategory, interests });

    await newInterestTab.save();

    return res.status(201).json({
      status: "success",
      message: "Interest Tab created successfully.",
      data: newInterestTab,
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: "error",
      error: "An error occurred while creating the Interest Tab.",
    });
  }
};

// Get all Interest Tabs
exports.getAllInterestTabs = async (req, res) => {
  try {
    const interestTabs = await InterestTab.find();

    if (!interestTabs.length) {
      return res.status(404).json({
        status: "fail",
        message: "No Interest Tabs found.",
      });
    }

    return res.status(200).json({
      status: "success",
      data: interestTabs,
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: "error",
      error: "An error occurred while retrieving the Interest Tabs.",
    });
  }
};

// Get a specific Interest Tab by category
exports.getInterestTabByCategory = async (req, res) => {
  try {
    const { topCategory } = req.params;

    const interestTab = await InterestTab.findOne({ topCategory });

    if (!interestTab) {
      return res.status(404).json({
        status: "fail",
        message: `Interest Tab with category '${topCategory}' not found.`,
      });
    }

    return res.status(200).json({
      status: "success",
      data: interestTab,
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: "error",
      error: "An error occurred while retrieving the Interest Tab.",
    });
  }
};

// Update the interests in an existing Interest Tab by category
exports.updateInterestTab = async (req, res) => {
  try {
    const { topCategory } = req.params;
    const { interests } = req.body;

    // Check if interests are provided as an array
    if (!interests || !Array.isArray(interests)) {
      return res.status(400).json({
        status: "fail",
        error: "Interests must be provided as an array.",
      });
    }

    // Find the InterestTab by topCategory
    const interestTab = await InterestTab.findOne({ topCategory });

    if (!interestTab) {
      return res.status(404).json({
        status: "fail",
        message: `Interest Tab with category '${topCategory}' not found.`,
      });
    }

    // Get the titles of the current interests in the interestTab
    const currentInterestsTitles = interestTab.interests.map(
      (interest) => interest.title
    );

    // Filter out new interests that already exist in the current interests based on the 'title'
    const newInterests = interests.filter((newInterest) => {
      return (
        newInterest.title && !currentInterestsTitles.includes(newInterest.title)
      );
    });

    // Add the new interests to the existing ones
    interestTab.interests = [...interestTab.interests, ...newInterests];

    // Save the updated InterestTab
    await interestTab.save();

    return res.status(200).json({
      status: "success",
      message: "Interest Tab updated successfully.",
      data: interestTab,
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: "error",
      error: "An error occurred while updating the Interest Tab.",
    });
  }
};

// Delete an Interest Tab by category
exports.deleteInterestTab = async (req, res) => {
  try {
    const { topCategory } = req.params;

    const interestTab = await InterestTab.findOneAndDelete({ topCategory });

    if (!interestTab) {
      return res.status(404).json({
        status: "fail",
        message: `Interest Tab with category '${topCategory}' not found.`,
      });
    }

    return res.status(200).json({
      status: "success",
      message: "Interest Tab deleted successfully.",
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: "error",
      error: "An error occurred while deleting the Interest Tab.",
    });
  }
};
