const express = require("express");
const passport = require("passport");
const { Strategy: GoogleStrategy } = require("passport-google-oauth20");
const AppError = require("../utils/appError");
const User = require("../models/userModel");
const authController = require("../controllers/authController");
const router = express.Router();

// **Public Routes** (no authentication required)
router.post("/signup-name", authController.signupName); // Step 1: Name (First and Last Name)
router.post("/signup-email/:tempUserId", authController.signupEmail); // Step 2: Email
router.post("/signup-password/:tempUserId", authController.signupPassword); // Step 3: Password
router.post(
  "/signup-profileInfo/:tempUserId",
  authController.signupProfileInfo
); // Step 4: Profile Info
router.post("/signup-interests/:tempUserId", authController.signupInterests); // Step 5: Interests
router.post("/signup-addPhoto/:tempUserId", authController.signupPhoto); // Step 6: Add Photo (optional)
router.post("/resend-otp/:tempUserId", authController.resendOTP);
router.post("/signup-verifyOTP/:tempUserId", authController.verifyOTP); // Step 7: Verify OTP
router.post("/login", authController.login);
router.get("/logout", authController.logout);
// Route to update the current user's password
router.patch(
  "/updateMyPassword",
  authController.protect,
  authController.updatePassword
);
// Google OAuth2.0 Strategy Configuration
passport.use(
  new GoogleStrategy(
    {
      clientID: process.env.CLIENT_ID,
      clientSecret: process.env.CLIENT_SECRET,
      callbackURL:
        process.env.CALLBACK_URL ||
        "http://localhost:3000/auth/google/callback",
    },
    (accessToken, refreshToken, profile, done) => {
      // Save user profile in session or database here
      return done(null, profile);
    }
  )
);

// Serialize user to session
passport.serializeUser((user, done) => {
  done(null, user); // Store the user in the session
});

passport.deserializeUser((user, done) => {
  done(null, user); // Retrieve the user from the session
});

const isAuthenticated = (req, res, next) => {
  if (!req.isAuthenticated()) {
    return next(new AppError("Not authenticated. Please log in.", 401));
  }
  next();
};

router.get("/google/sign-in", (req, res) => {
  res.send("<a href='/auth/google'>Sign in with Google</a>");
});

// Redirect to Google for authentication
router.get(
  "/google",
  passport.authenticate("google", {
    scope: ["profile", "email"], // Specify the scope for Google authentication
  })
);

// Google callback route
router.get(
  "/google/callback",
  passport.authenticate("google", { failureRedirect: "/" }), // Redirect to "/" on failure
  async (req, res, next) => {
    try {
      // Extract user data from req.user (populated by passport)
      const { displayName, emails, photos, provider, id } = req.user;

      // Check if the user already exists in the database
      const existingUser = await User.findOne({ email: emails[0].value });
      if (existingUser) {
        // If the user exists, log them in and redirect them to a dashboard or profile page
        return res.redirect("/dashboard"); // Or your desired page after login
      }

      // Create a new user document
      const newUser = await User.create({
        firstName: req.user.name.givenName, // Use provided data
        lastName: req.user.name.familyName, // Use provided data
        email: emails[0].value, // Use the first email
        photo: photos[0].value, // Use the first photo URL
        role: "user", // Default role
        provider: provider, // Store provider (Google in your case)
        password: "password123", // You should handle password generation differently for social login
        passwordConfirm: "password123", // Same as above
        recentJob: "Unspecified", // Example, you can modify as needed
        company: "Unspecified", // Example, you can modify as needed
      });

      // Only redirect after successful creation, not before
      return res.redirect("/"); // Or any other page you want after successful signup
    } catch (err) {
      return next(
        new AppError(
          err.message || "An error occurred while saving user data",
          500
        )
      );
    }
  }
);

router.get("/user-data", isAuthenticated, (req, res) => {
  res.status(200).json({
    success: true,
    user: req.user, // Ensure this contains user data
  });
});
// // Logout API for Google Sign-In
// router.get("/logout", (req, res, next) => {
//   // Log the user out
//   req.logout((err) => {
//     if (err) {
//       return next(new AppError("Error logging out", 500)); // Handle logout errors
//     }

//     // Clear the session
//     req.session.destroy((sessionErr) => {
//       if (sessionErr) {
//         return next(new AppError("Error clearing session", 500)); // Handle session clear errors
//       }

//       // Send success response
//       res.status(200).json({
//         status: "success",
//         message: "Logged out successfully",
//       });
//     });
//   });
// });

module.exports = router;
