const express = require("express");
const passport = require("passport");
const { Strategy: LinkedInStrategy } = require("passport-linkedin-oauth2");
const AppError = require("../utils/appError");
const User = require("../models/userModel");

const router = express.Router();

// LinkedIn OAuth2.0 Strategy Configuration
passport.use(
  new LinkedInStrategy(
    {
      clientID: process.env.LINKEDIN_CLIENT_ID, // Use the client ID from .env
      clientSecret: process.env.LINKEDIN_CLIENT_SECRET, // Use the client secret from .env
      callbackURL:
        process.env.LINKEDIN_CALLBACK_URL ||
        "http://localhost:3000/auth-linkedin/linkedin/callback",
      scope: ["r_liteprofile"], // Modify the scope based on your app's permission
      state: true, // Optional: to prevent CSRF
    },
    async (accessToken, refreshToken, profile, done) => {
      // Handle LinkedIn profile data
      return done(null, profile);
    }
  )
);

// Serialize user to session
passport.serializeUser((user, done) => {
  done(null, user);
});

// Deserialize user from session
passport.deserializeUser((user, done) => {
  done(null, user);
});

// Middleware to check if user is authenticated
const isAuthenticated = (req, res, next) => {
  if (!req.isAuthenticated()) {
    return next(new AppError("Not authenticated. Please log in.", 401));
  }
  next();
};

// LinkedIn sign-in route (landing page with sign-in link)
router.get("/linkedin/sign-in", (req, res) => {
  res.send("<a href='/auth-linkedin/linkedin'>Sign in with LinkedIn</a>");
});

// Redirect to LinkedIn for authentication
router.get(
  "/linkedin",
  passport.authenticate("linkedin", {
    scope: ["r_emailaddress", "r_liteprofile"], // Define the scope for LinkedIn authentication
  })
);

// LinkedIn callback route
router.get(
  "/linkedin/callback",
  passport.authenticate("linkedin", {
    failureRedirect: "/auth/linkedin/sign-in",
  }), // Redirect to sign-in page on failure
  async (req, res, next) => {
    try {
      if (!req.user) {
        return next(
          new AppError("Authentication failed, please try again.", 401)
        );
      }

      // Extract user data from LinkedIn profile
      const { displayName, emails, photos, provider, id } = req.user;

      // Check if the user exists in the database
      const existingUser = await User.findOne({ email: emails[0].value });
      if (existingUser) {
        return res.redirect("/dashboard"); // Redirect to dashboard if the user exists
      }

      // If user does not exist, create a new user record
      const newUser = await User.create({
        firstName: req.user.firstName,
        lastName: req.user.lastName,
        email: emails[0].value,
        photo: photos[0].value,
        role: "user",
        provider: provider,
        password: "password123", // Use a default or random password generation method for social logins
        passwordConfirm: "password123",
        recentJob: "Unspecified",
        company: "Unspecified",
      });

      return res.redirect("/welcome"); // Redirect to the welcome page after sign-up
    } catch (err) {
      return next(new AppError("Error during user data processing.", 500));
    }
  }
);

// User data route (Only accessible if authenticated)
router.get("/user-data", isAuthenticated, (req, res) => {
  res.status(200).json({
    success: true,
    user: req.user, // Ensure this contains the user data
  });
});

// Logout route for LinkedIn Sign-In
router.get("/logout", (req, res, next) => {
  // Log the user out
  req.logout((err) => {
    if (err) {
      return next(new AppError("Error logging out", 500)); // Handle logout errors
    }

    // Clear the session
    req.session.destroy((sessionErr) => {
      if (sessionErr) {
        return next(new AppError("Error clearing session", 500)); // Handle session clear errors
      }

      // Send a success response
      res.status(200).json({
        status: "success",
        message: "Logged out successfully",
      });
    });
  });
});

module.exports = router;
